/*
   Yamaha Song Display

   Get the song details from the R-N602 and display them.
   Assumes a 2x16 LCD display.

 */

// Configuration
// serverAddress is the IP address of the R-N602
const char serverAddress[] =  "192.168.0.22";
// trackDelay is the length of time the track is displayed in ms
const unsigned int trackDelay = 10000;
// albumDelay is the length of time the album and artist are displayed in ms
const unsigned int artistDelay = 5000;
// scrollDelay is the length of time between shifts when scrolling in ms
const unsigned int scrollDelay = 400;
// End of Configuration 
// Do not change below this line

// JSON
#include <ArduinoHttpClient.h>
#include <ArduinoJson.h>

// LCD
#include <LiquidCrystal.h>;

LiquidCrystal lcd(8, 9, 4, 5, 6, 7); // select the pins used on the LCD panel

// Wi-Fi
#include <WiFiS3.h>

int status = WL_IDLE_STATUS;
#include "arduino_secrets.h" 
char ssid[] = SECRET_SSID;    // the network SSID (name)
char pass[] = SECRET_PASS;    // the network password

int port = 80;

WiFiClient wifiClient;
HttpClient client = HttpClient(wifiClient, serverAddress, port);

int alternate = 0; // used to alternate between artist/album and track
const int LCDwidth = 16;
const int maxItemLength = 64;

void printWifiStatus() {
   // print the SSID of the network you're attached to:
   Serial.print("SSID: ");
   Serial.println(WiFi.SSID());

   // print your board's IP address:
   IPAddress ip = WiFi.localIP();
   Serial.print("IP Address: ");
   Serial.println(ip);

   // print the received signal strength:
   long rssi = WiFi.RSSI();
   Serial.print("signal strength (RSSI):");
   Serial.print(rssi);
   Serial.println(" dBm");
}

void scrollText(int line, char * text) {
   char lineBuf[LCDwidth + 1];
   int len = strlen(text);

   if (len > LCDwidth) {
      for (int i = 0; i < len - LCDwidth + 1; i++) { 
         lcd.setCursor(0, line); 
         memset(lineBuf, 0, sizeof(lineBuf));
         strncpy(lineBuf, text + i, sizeof(lineBuf) - 1);
         lcd.print(lineBuf); 
         delay(scrollDelay); 
      } 
   } 

   lcd.setCursor(0, line); 
   memset(lineBuf, 0, sizeof(lineBuf));
   strncpy(lineBuf, text, sizeof(lineBuf) - 1);
   lcd.print(lineBuf);
   delay(scrollDelay); 
}

void getSongDetails() {
   char line1[maxItemLength + 1];
   char line2[maxItemLength + 1];
   char trackBuf[maxItemLength + 1];
   int trackLines;

   if (WiFi.status() == WL_CONNECTED) {
      client.get("/YamahaExtendedControl/v1/netusb/getPlayInfo"); 
      int statusCode = client.responseStatusCode();
      String payload = client.responseBody();

      if (statusCode != 200) { // 200 denotes success
         Serial.print("Status code: ");
         Serial.println(statusCode);
      }

      // Create a JSON document to store the response
      StaticJsonDocument<1024> doc;
      DeserializationError error = deserializeJson(doc, payload);

      if (error) {
         Serial.print(F("deserializeJson() failed: "));
         Serial.println(error.f_str());
         return;
      }

      // Extract the values
      const char* artist = doc["artist"];
      const char* track = doc["track"];
      const char* album = doc["album"];

      // display the values
      lcd.setCursor(0,0); 
      lcd.clear();
      memset(line1, 0, sizeof(line1));
      memset(line2, 0, sizeof(line2));
      if (++alternate % 2 == 0) {
         strncpy(line1, artist, maxItemLength);
         scrollText(0, line1);
         strncpy(line2, album, maxItemLength);
         scrollText(1, line2);
         delay(artistDelay); 
      } else {
         trackLines = strlen(track) > maxItemLength ? maxItemLength : strlen(track);
         memset(trackBuf, 0, sizeof(trackBuf));
         strncpy(trackBuf, track, trackLines);
         strncpy(line1, trackBuf, LCDwidth);
         lcd.setCursor(0,0); 
         lcd.print(line1);
         strncpy(line2, trackBuf + LCDwidth, maxItemLength - LCDwidth); 
         scrollText(1, line2);

         delay(trackDelay); 
      }

      // Print the values
      Serial.print("Artist: ");
      Serial.println(artist ? artist : "N/A");
      Serial.print("Track Title: ");
      Serial.println(track ? track : "N/A");
      Serial.print("Album: ");
      Serial.println(album ? album : "N/A");
   }
}

void setup() {
   // Open serial communications and wait for port to open:
   Serial.begin(9600);
   while (!Serial) {
      ; // wait for serial port to connect. 
   }

   // LCD
   lcd.begin(LCDwidth, 2); 
   lcd.setCursor(0,0); 
   lcd.print("Yamaha Song v1.0");

   // attempt to connect to WiFi network:
   while (WiFi.status() != WL_CONNECTED) {
      lcd.setCursor(0,1); 
      lcd.print("Connecting ...");
      Serial.print("Attempting to connect to SSID: ");
      Serial.println(ssid);
      WiFi.begin(ssid, pass);

      // wait 10 seconds for connection:
      delay(10000);
   }

   Serial.println("Connected to WiFi");
   lcd.setCursor(0,1); 
   lcd.print("Connected       ");

   printWifiStatus();
}

void loop() {
   getSongDetails();
}

